Attribute VB_Name = "AB30XV1"
Option Explicit
Const MB_ICONEXCLAMATION = 48   ' Warning message
Const MB_ICONSTOP = 16          ' Critical message

Dim ErrorStatus%    'Error: bit 7 is set; otherwise bit 7 is 0.
Dim StatusByteB$     'status byte from CM11X
Dim CancelByteB$     'CancelByteB$ is "24"
Dim HighByte%, LowByte%

 Function ABBaud%(Baud%, PortNo&, abPort As Control, Timeout%)
' To disable Timeout% (in seconds), set Timeout% to 0
' Timeout% will be set to 1hr if >1hr.
'
' Sets the filter wheel's RS232 baud rate and changes the Com port's
' baud rate to match.
'
' This function requires ABInitComPort() to be present.
'
' IMPORTANT!  The filter wheel baud rate will change immediately after the
'             status and cancel bytes are sent.  The controlling program
'             must also switch baud rates to follow the change.  This
'             function performs that change.
'
'             The filter wheel will remember the last baud rate when turned
'             off.  Any further communication must match this saved baud.
'
' Command: To Instrument:   <58><Baud Code Byte>
'          From Instrument: <Status Byte><24>
'
' This function's return value is an INTEGER.  A value of zero at bit 7
' (the MSB) indicates success.
'
' The passed parameters Baud%, PortNo&, and Timeout% are INTEGERs.
' abPort is a commnunication control.
'
' Values of Baud% are 9600, 4800, 2400, 1200, 600, 300, 150, and 75.
'
' Valid choices for PortNo& are 1 = COM1, 2 = COM2, 3 = COM3, 4 = COM4.
'
' abPort is the communications control assigned to the filter wheel's
' COM port by the ABInitComPort%() function. It may be changed in this
' function when the Com port baud rate is changed.
'
  Dim BaudByte%, Status%
  On Error GoTo ABBaudErr
  ErrorStatus% = 0    'No error

  Select Case Baud%
    Case 9600
      BaudByte% = 0
    Case 4800
      BaudByte% = 1
    Case 2400
      BaudByte% = 2
    Case 1200
      BaudByte% = 3
    Case 600
      BaudByte% = 4
    Case 300
      BaudByte% = 5
    Case 150
      BaudByte% = 6
    Case 75
      BaudByte% = 7
    Case Else
      ErrorStatus% = 128      'Error; set bit 7 of return value
      MsgBox "The entered number is out of range", MB_ICONEXCLAMATION, "AB.BAS"
  End Select

  If ErrorStatus% = 0 Then
    abPort.Output = Chr$(58) + Chr$(BaudByte%)
    ' Get Status and Cancel Bytes
    ErrorStatus% = ABGetStatusCancel%(abPort, Timeout%)   'Timeout% in seconds

    ' Change the com port baud rate to match the filter wheel
    Status% = ABInitComPort%(PortNo&, Baud%, abPort)
    
    If Status% <> 0 Then ErrorStatus% = Status%
  End If

ABBaudResume:
  ABBaud% = ErrorStatus%
  Exit Function

ABBaudErr:
  MsgBox "Error code is : " & Err, MB_ICONEXCLAMATION, "AB.BAS"
  ErrorStatus% = 128      'Error; set bit 7 of return value
  Resume ABBaudResume

End Function

Function ABEcho%(abPort As Control, Timeout%)
' To disable Timeout% (in seconds), set Timeout% to 0
' Timeout% will be set to 1hr if >1hr.
'
' Verifies communication by sending a single byte and receiving that same
' byte back.
'
' Command: To Instrument:   <27>
'          From Instrument: <27>
'
' This function's return value is an INTEGER.  A value of zero at bit 7
' (the MSB) indicates success.
'
' The passed parameter abPort is a control, and Timeout% as an integer.
'
' abPort is the communications control assigned to the filter wheel's
' COM port by the ABInitComPort%() function.
'
  Dim DelayTime%, ByteB$, Minutes%
  On Error GoTo ABEchoErr
  abPort.Output = Chr$(27)
  ' Get Echo Byte
  If Timeout% > 0 Then
    ErrorStatus% = ABTimeout%(abPort, Timeout%)
    If ErrorStatus% = 128 Then GoTo ABEchoResume
  End If
'
  ByteB$ = abPort.Input          'Read in byte
  If Asc(ByteB$) = 27 Then
    ErrorStatus% = 0            'no error
  Else
    ErrorStatus% = 128          'Error; set bit 7
  End If
'
ABEchoResume:
  ABEcho% = ErrorStatus%
  Exit Function
'
ABEchoErr:
  MsgBox "Error code is : " & Err, MB_ICONEXCLAMATION, "AB.BAS"
  ErrorStatus% = 128          'Error; set bit 7
  Resume ABEchoResume

End Function

Function ABEepromRead%(Address%, DataWord&, abPort As Control, Timeout%)
' To disable Timeout% (in seconds), set Timeout% to 0
' Timeout% will be set to 1hr if >1hr.
'
' Reads a byte from the filter wheel's non-volatile memory.
'
' Command: To Instrument:   <56><Address Byte>
'          From Instrument: <Data Byte><Status Byte><24>
'
' abPort as a communication control.
' The passed parameters Address%, Timeout% are intergers.
' DataByte% contains a returned byte.
'
' This function's returned value is an INTEGER.  A value of zero at bit 7
' (the MSB) indicates success.
'
' Valid choices for Address% are 0 through 127.
'
' abPort is the communications control assigned to the filter wheel's
' COM port by the ABInitComPort%() function.
'
  Dim High&, Low&, DataHighByteB$, DataLowByteB$
  On Error GoTo ABEepromReadErr
  If ((Address% >= 0) And (Address% <= 63)) Then
    abPort.Output = Chr$(56) + Chr$(Address%)

    'Checking Timeout% for the first byte.
    If Timeout% > 0 Then
      ErrorStatus% = ABTimeout%(abPort, Timeout%)
      If ErrorStatus% = 128 Then GoTo ABEepromReadResume
    End If
    
    'Get returned novram data
    DataHighByteB$ = abPort.Input            'Read in data byte
    High& = Asc(DataHighByteB$)
    If Timeout% > 0 Then
      ErrorStatus% = ABTimeout%(abPort, Timeout%)
      If ErrorStatus% = 128 Then GoTo ABEepromReadResume
    End If
    DataLowByteB$ = abPort.Input             'Read in data byte
    Low& = Asc(DataLowByteB$)

    DataWord& = High& * 256 + Low&

    
    'Get Status and Cancel Bytes
    ErrorStatus% = ABGetStatusCancel%(abPort, Timeout%)
  Else
    ErrorStatus% = 128
    MsgBox "The entered number is out of range", MB_ICONEXCLAMATION, "AB.BAS"
  End If

ABEepromReadResume:
  ABEepromRead% = ErrorStatus%
  Exit Function

ABEepromReadErr:
  MsgBox "Error code is : " & Err, MB_ICONEXCLAMATION, "AB.BAS"
  ErrorStatus% = 128      'Error; set bit 7 of return value
  Resume ABEepromReadResume

End Function

Function ABEepromWrite%(Address%, DataWord&, abPort As Control, Timeout%)
' To disable Timeout% (in seconds), set Timeout% to 0
' Timeout% will be set to 1hr if >1hr.
'
' Writes a byte to the filter wheel's non-volatile memory.
'
' WARNING!!!  Improper use of this command may corrupt the configuration
'             of the filter wheel.  DO NOT use this command if you are
'             unsure of the implications!
'
' Command: To Instrument:   <59><Address Byte><Data High Byte><Data Low Byte>
'                           <Checksum Byte>
'          From Instrument: <Status Byte><24>
'
' abPort is a communication control.
' The passed parameters Address%, Timeout% are intergers. DataWord& is a long integer.
'
' This function's returned value is an INTEGER.  A value of zero at bit 7
' (the MSB) indicates success.
'
' Valid choices for Address% are 0 through 63.
'
' Valid values for DataWord& are 0 through 65535.
'
' abPort is the communications control assigned to the filter wheel's
' COM port by the ABInitComPort%() function.
'
  Dim Checksum%
  On Error GoTo ABEepromWriteErr
  If ((Address% >= 0) And (Address% <= 63) And (DataWord& >= 0) And (DataWord& <= 65535)) Then
    ' Calculate the 2 byte wavelength specifier to send in the command
    HighByte% = Int(DataWord& / 256!)
    LowByte% = Int(DataWord& - 256! * HighByte%)
    
    Checksum% = Address% + HighByte% + LowByte%
    If Checksum% > 255 Then
      Checksum% = Checksum% - 256     'truncate to 1 byte long
    End If
    abPort.Output = Chr$(59) + Chr$(Address%)
    abPort.Output = Chr$(HighByte%) + Chr$(LowByte%) + Chr$(Checksum%)

' Get Status and Cancel Bytes
    ErrorStatus% = ABGetStatusCancel%(abPort, Timeout%)
  Else
    ErrorStatus% = 128      'Error; set bit 7 of return value
    MsgBox "The entered number is out of range", MB_ICONEXCLAMATION, "AB.BAS"
  End If

ABEepromWriteResume:
  ABEepromWrite% = ErrorStatus%
  Exit Function

ABEepromWriteErr:
  MsgBox "Error code is : " & Err, MB_ICONEXCLAMATION, "CM.BAS"
  ErrorStatus% = 128      'Error; set bit 7 of return value
  Resume ABEepromWriteResume

End Function

Function ABFilter%(FilterN%, FilterMax%, abPort As Control, Timeout%)
' To disable Timeout% (in seconds), set Timeout% to 0
' Timeout% will be set to 1hr if >1hr.
'
' Moves the filter wheel to a desired filter.
'
' Command: To Instrument:   <15><Position Byte>
'          Action:          Filter wheel moves to specified position
'          From Instrument: <Status Byte><24>
'
' This function's return value is an INTEGER.  A value of zero at bit 7
' (the MSB) indicates success.
'
' The passed parameter FilterN% is an INTEGER and abPort as commnunication
' control.
'
' FilterN% may range from rom -32,768 to 32,767, but acceptable values to
' the filter wheel will depend on the filter wheel model.
' FilterMax% is the maximum number of filters.
' (AB301: 6, AB302: 5, AB303: 12)
'
' abPort is the communications control assigned to the filter wheel's
' COM port by the ABInitComPort%() function.
'
  On Error GoTo ABFilterErr
  If (FilterN% >= 1) And (FilterN% <= FilterMax%) Then

  abPort.Output = Chr$(15)
  abPort.CTSTimeout = 100   ' Sets the number of milliseconds to wait for the CTS signal
  abPort.Output = Chr$(FilterN%)

  ' Get Status and Cancel Bytes
  ErrorStatus% = ABGetStatusCancel%(abPort, Timeout%)   'Timeout% in seconds
  Else
    ErrorStatus% = 128      'Error; set bit 7 of return value
    MsgBox "The entered number is out of range", MB_ICONEXCLAMATION, "AB.BAS"
  End If

ABFilterResume:
  ABFilter% = ErrorStatus%
  Exit Function

ABFilterErr:
  MsgBox "Error code is : " & Err, MB_ICONEXCLAMATION, "AB.BAS"
  ErrorStatus% = 128      'Error; set bit 7 of return value
  Resume ABFilterResume

End Function

Function ABGetStatusCancel%(abPort As Control, Timeout%)
' To disable Timeout% (in seconds), set Timeout% to 0
' Timeout% will be set to 1hr if >1hr.
'
' Gets the status and cancel bytes sent back from the filter wheel. This
' function is used by other filter wheel library functions and is intended
' to be used elsewhere.
'
' This function returns an INTEGER that is value of the filter wheel's
' status byte.
'
' abPort is the communications control assigned to the filter wheel's
' COM port by the CMInitComPort%() function.
'
'
  Dim DelayTime%, Minutes%, TimeoutErr%
  On Error GoTo ABGetStatusCancelErr
  ' Get Status Byte
  If Timeout% > 0 Then
    TimeoutErr% = ABTimeout%(abPort, Timeout%)
    If TimeoutErr% = 128 Then
      ErrorStatus% = TimeoutErr%
      GoTo ABGetStatusCancelResume
    End If
  End If
  
  StatusByteB$ = abPort.Input              'Read in status byte
  ErrorStatus% = Asc(StatusByteB$)         'Convert from char to integer

  ' Get Cancel Byte
  If Timeout% > 0 Then
    TimeoutErr% = ABTimeout%(abPort, Timeout%)
    If TimeoutErr% = 128 Then
      ErrorStatus% = TimeoutErr%
      GoTo ABGetStatusCancelResume
    End If
  End If
  CancelByteB$ = abPort.Input              'Read in final byte
  If Asc(CancelByteB$) <> 24 Then ErrorStatus% = 128

ABGetStatusCancelResume:
  ABGetStatusCancel% = ErrorStatus%
  Exit Function

ABGetStatusCancelErr:
  MsgBox "Error code is : " & Err, MB_ICONEXCLAMATION, "AB.BAS"
  ErrorStatus% = 128      'Error; set bit 7 of return value
  Resume ABGetStatusCancelResume

End Function

Function ABInitComPort%(PortNo&, Baud%, abPort As Control)
' To disable Timeout% (in seconds), set Timeout% to 0
' Timeout% will be set to 1hr if >1hr.
'
' Opens a RS232 com port to communicate with the filter wheel.  8 data
' bits, 1 stop bit, and no parity are fixed.  Baud rate is selectable,
' as is the COM port to use.  The baud rate selected here must match the
' filter wheel's baud rate (selected with the BAUD command.)
'
' This function must be run before any other filter wheel library function
' calls.
'
' This function's return value is an INTEGER.  A value of zero indicates
' success.  Non-zero return values indicate an error has occured.
'
' The passed parameters PortNo&, and Baud% all INTEGERs. abPort as a
' commnunications Control.
'
' Valid choices for PortNo& are 1 = COM1, 2 = COM2, 3 = COM3, 4 = COM4.
'
' Valid baud rates for AB30X are 9600, 4800, 2400, 1200, 600, 300, 150 and 75,
' but valid baud rates for Visual Basic are 9600, 4800, 2400, 1200, 600, 300.
'
  Dim InS$
  On Error GoTo ABInitComPortErr
  If abPort.PortOpen Then
    abPort.PortOpen = False
  End If
  abPort.CommPort = PortNo&
  abPort.Settings = Baud% & ",N,8,1"
  abPort.RTSEnable = True             ' Enables the Request To Send line
  abPort.InputLen = 1                 ' Read 1 character at the receive buffer. 0: entire contents.
  abPort.PortOpen = True

  ErrorStatus% = 0                      ' No error

ABInitComPortResume:
  ABInitComPort% = ErrorStatus%
  Exit Function

ABInitComPortErr:
  MsgBox "Error code is : " & Err, MB_ICONEXCLAMATION, "AB.BAS"
  ErrorStatus% = 128      'Error; set bit 7 of return value
  Resume ABInitComPortResume

End Function

Function ABQueryPos%(Position%, abPort As Control, Timeout%)
' To disable Timeout% (in seconds), set Timeout% to 0
' Timeout% will be set to 1hr if >1hr.
'
' QUERY POSITION will return the present filter position is "Position%"
' (Eg: 1, 2, 3, ... decimal, not ASCII character "1", "2", "3".)
'
' Command: To Instrument:   <29>
'          From Instrument: <Position Byte>
'                           <Status Byte><24>
'          Action:          None
'
' This function's return value is an INTEGER.  A value of zero at bit 7
' (the MSB) indicates success.
'
' The passed abPort as Control. The return value, Position%, is an INTEGER.
'
' abPort is the communications control assigned to the filter wheel's
' COM port by the ABInitComPort%() function.
'
  Dim DataPos$
  Dim High&, Low&
  On Error GoTo ABQueryPosErr
  
  abPort.Output = Chr$(29)
  'Checking Timeout% for the first byte.
  If Timeout% > 0 Then
    ErrorStatus% = ABTimeout%(abPort, Timeout%)
    If ErrorStatus% = 128 Then GoTo ABQueryPosResume
  End If
  'Get returned queried data
  DataPos$ = abPort.Input            'Read in data byte
  Position% = Asc(DataPos$)
  If Timeout% > 0 Then
    ErrorStatus% = ABTimeout%(abPort, Timeout%)
    If ErrorStatus% = 128 Then GoTo ABQueryPosResume
  End If

  'Get Status and Cancel Bytes
  ErrorStatus% = ABGetStatusCancel%(abPort, Timeout%)

ABQueryPosResume:
  ABQueryPos% = ErrorStatus%
  Exit Function

ABQueryPosErr:
  MsgBox "Error code is : " & Err, MB_ICONEXCLAMATION, "AB.BAS"
  ErrorStatus% = 128      'Error; set bit 7 of return value
  Resume ABQueryPosResume

End Function

Function ABReset%(abPort As Control)
'
' This command effectively does a power-on reset, resetting the controller
' board and rehoming the wheel.
'
' Command: To Instrument:   <255><255>
'          Action:          Filter wheel re-homes and goes to filter
'                           position 1.
'          From controller: Nothing.
'
' This function's return value is an INTEGER.  A value of zero at bit 7
' (the MSB) indicates success.
'
' The passed parameter abPort is a control.
'
' abPort is the communications control assigned to the filter wheel's
' COM port by the ABInitComPort%() function.
'
  On Error GoTo ABResetErr
  ErrorStatus% = 0
  abPort.Output = Chr$(255) + Chr$(255)

ABResetResume:
  ABReset% = ErrorStatus%
  Exit Function

ABResetErr:
  MsgBox "Error code is : " & Err, MB_ICONEXCLAMATION, "AB.BAS"
  ErrorStatus% = 128      'Error; set bit 7 of return value
  Resume ABResetResume

End Function

Function ABRtsCts%(abPort As Control, RtsOutPc%, CtsInPc%)
' RtsOutPc% is true if the PC enables the RTS line, and vive-versa.
' ClsInPc% is true if CTS line is high, and vice-versa
  On Error GoTo ABRtsCtsErr
  ErrorStatus% = 0      ' No error.
  RtsOutPc% = abPort.RTSEnable
  CtsInPc% = abPort.CTSHolding

ABRtsCtsResume:
  ABRtsCts% = ErrorStatus%
  Exit Function

ABRtsCtsErr:
  MsgBox "Error code is : " & Err, MB_ICONEXCLAMATION, "AB.BAS"
  ErrorStatus% = 128      'Error; set bit 7 of return value
  Resume ABRtsCtsResume

End Function

Function ABStepDn%(abPort As Control, Timeout%)
' To disable Timeout% (in seconds), set Timeout% to 0
' Timeout% will be set to 1hr if >1hr.
'
' Used to fine adjust the current position. For filter one, it may be
' used preceding a ZERO command to set the default filter one position.
'
' Command: To Instrument:   <1>
'          Action:          Wheel moves one motor step toward next lower position
'          From Instrument: <Status Byte><24>
'
' This function's return value is an INTEGER.  A value of zero at bit 7
' (the MSB) indicates success.
'
' The passed parameter abPort is a commnunication control.
'
' abPort is the communications control assigned to the filter wheel's
' COM port by the ABInitComPort%() function.
'
  On Error GoTo ABStepDnErr
  abPort.Output = Chr$(1)
  'Get Status and Cancel Bytes
  ErrorStatus% = ABGetStatusCancel%(abPort, Timeout%)

ABStepDnResume:
  ABStepDn% = ErrorStatus%
  Exit Function

ABStepDnErr:
  MsgBox "Error code is : " & Err, MB_ICONEXCLAMATION, "AB.BAS"
  ErrorStatus% = 128      'Error; set bit 7 of return value
  Resume ABStepDnResume

End Function

Function ABStepUp%(abPort As Control, Timeout%)
' To disable Timeout% (in seconds), set Timeout% to 0
' Timeout% will be set to 1hr if >1hr.
'
' Used to fine adjust the current position. For filter one, it may be
' used preceding a ZERO command to set the default filter one position.
'
' Command: To Instrument:   <7>
'          Action:          Wheel moves one motor step toward next higher position
'          From Instrument: <Status Byte><24>
'
' This function's return value is an INTEGER.  A value of zero at bit 7
' (the MSB) indicates success.
'
' The passed parameter abPort is a commnunication control.
'
' abPort is the communications control assigned to the filter wheel's
' COM port by the ABInitComPort%() function.
'
  On Error GoTo ABStepUpErr
  abPort.Output = Chr$(7)
  'Get Status and Cancel Bytes
  ErrorStatus% = ABGetStatusCancel%(abPort, Timeout%)

ABStepUpResume:
  ABStepUp% = ErrorStatus%
  Exit Function

ABStepUpErr:
  MsgBox "Error code is : " & Err, MB_ICONEXCLAMATION, "AB.BAS"
  ErrorStatus% = 128      'Error; set bit 7 of return value
  Resume ABStepUpResume

End Function

Function ABTimeout%(abPort As Control, Timeout%)
' To disable Timeout% (in seconds), set Timeout% to 0
' Timeout% will be set to 1hr if >1hr.
'
' This function will return 128 (bit 7 sets 1) if during the time was set
' by Timeout% but no byte was received on the transmission line. Otherwise
' the return number is 0.
'
' abPort is the communications control assigned to the filter wheel's
' COM port by the ABInitComPort%() function.
'
  Dim Minutes%, DelayTime%
  If Timeout% > 0 Then
    If Timeout% > 3599 Then Timeout% = 3599
    Minutes% = Minute(Time$)
    DelayTime% = Second(Time$) + Timeout%     'Timeout% in seconds
    While (abPort.InBufferCount = 0) And ((Minute(Time$) - Minutes%) * 60 + Second(Time$) < DelayTime%)
    Wend    'Wait until character present or waiting time is less than delay time
    If abPort.InBufferCount = 0 Then
      'MsgBox "Errors in the serial commnuncation line", MB_ICONSTOP, "CM.BAS"
      ABTimeout% = 128
    Else
      ABTimeout% = 0
    End If
  End If

End Function

Function ABZero%(abPort As Control, Timeout%)
' To disable Timeout% (in seconds), set Timeout% to 0
' Timeout% will be set to 1hr if >1hr.
'
' Sets the default filter one position. All other filter positions are
' derived from this location. The wheel must be set to filter one position
' prior to issuing this command.
'
' IMPORTANT! Failure to select filter one position prior to using this command
' may cause erratic or unpredictable results, including failure to operate.
' Care and discretion is advised.
'
' Command: To Instrument:   <52>
'          Action:          Current position is saved as filter one default position
'          From Instrument: <Status Byte><24>
'
' This function's return value is an INTEGER.  A value of zero at bit 7
' (the MSB) indicates success.
'
' abPort is the communications control assigned to the filter wheel's
' COM port by the ABInitComPort%() function.
'
  On Error GoTo ABZeroErr
  abPort.Output = Chr$(52)
  'Get Status and Cancel Bytes
  ErrorStatus% = ABGetStatusCancel%(abPort, Timeout%)

ABZeroResume:
  ABZero% = ErrorStatus%
  Exit Function

ABZeroErr:
  MsgBox "Error code is : " & Err, MB_ICONEXCLAMATION, "AB.BAS"
  ErrorStatus% = 128      'Error; set bit 7 of return value
  Resume ABZeroResume

End Function

